using System;
using System.Collections.Generic;
using System.Text;

namespace Framework
{
    public class ServiceManager
    {
        #region Member Variables

        private Dictionary<ServiceItem, ServiceItem> m_serviceDictionary = new Dictionary<ServiceItem, ServiceItem>();

        #endregion

        #region Constructions

        public ServiceManager() { }

        #endregion

        #region Operations

        public void Add<TService>(TService instance)
        {
            Add<TService, Object>(instance, null);
        }

        // user the key to distinguish the same type's differance instance
        public void Add<TService, TKey>(TService instance, TKey key)
        {
            ServiceItem service = new ServiceItem(typeof(TService), key, instance);

            m_serviceDictionary[service] = service;
        }

        public void Remove<TService>()
        {
            Remove<TService, Object>(null);
        }

        public void Remove<TService, TKey>(TKey key)
        {
            ServiceItem item = new ServiceItem(typeof(TService), key, null);
            if (m_serviceDictionary.ContainsKey(item))
                m_serviceDictionary.Remove(item);
        }

        public TServiceType Get<TServiceType>()
        {
            return Get<TServiceType, Object>(null);
        }

        public TServiceType Get<TServiceType, TKey>(TKey key)
        {
            TServiceType result = default(TServiceType);

            ServiceItem service = new ServiceItem(typeof(TServiceType), key, null);
            // The key not find, just return its default value
            if (!m_serviceDictionary.ContainsKey(service))
                return result;

            ServiceItem serviceFound = m_serviceDictionary[service];
            result = (TServiceType)serviceFound.ServiceInstance;

            return result;
        }

        #endregion

        #region Internal Classes

        private class ServiceItem
        {

            #region Member Varilables

            private Type m_serviceType;
            private Object m_serviceInstance;
            private Object m_key;

            #endregion

            #region Constructions

            public ServiceItem(Type serviceType, Object key, Object serviceInstance)
            {
                m_serviceType = serviceType;
                m_key = key;
                m_serviceInstance = serviceInstance;
            }

            public ServiceItem(Type serviceType, Object serviceInstance)
                : this(serviceType, null, serviceInstance)
            {
            }

            #endregion

            #region Properties

            public Type Type
            {
                get { return m_serviceType; }
            }

            public Object ServiceInstance
            {
                get { return m_serviceInstance; }
            }

            public Object Key
            {
                get { return m_key; }
            }

            #endregion

            #region Virtural Overrides

            // Only the Type and Key equal, then return true
            public override bool Equals(Object obj)
            {
                ServiceItem compareType = obj as ServiceItem;
                if (compareType == null)
                    return false;

                bool match = m_serviceType.Equals(compareType.m_serviceType);
                if (match && (m_key == compareType.m_key) && (m_key != null))
                    match = m_key.Equals(compareType.m_key);

                return match;
            }

            public override int GetHashCode()
            {
                int hashCode = m_serviceType.GetHashCode();
                if (m_key != null)
                    hashCode += m_key.GetHashCode();
                return hashCode;
            }

            #endregion
        }

        #endregion
    }
}
